/* lsrscan v 0.2 by Todd MacDermid */
/* It's been long enough... */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include "lsrscan.h"

extern int h_errno;
extern char *optarg;
extern int optind, opterr, optopt;

void usage(char *programName) {
  printf("usage: %s [-p dstport] [-s srcport] [-S ip] \n\t[-t (to|through|both)] <hostname ...>\n", programName);
  exit(0);
}

int sendLSR(struct packetInfo *settings) { 
  ip_t *ip;
  uint8_t *packetBuf;
  uint16_t packetSize;
  struct tcp_hdr *tcpHeader;
  int IPoptionSize;
  
  if((ip = ip_open()) == NULL) {
    fprintf(stderr, 
	    "Cannot initialize socket (not root or unsupported interface)\n");
    return(-1);
  }
  
  IPoptionSize = LSROVERHEAD + (settings->hopCount-2)*IP_ADDR_LEN;
  if (IPoptionSize%4)
    IPoptionSize += (4-(IPoptionSize%4)); /* Pad out to word boundary */
  if(IPoptionSize > IP_OPT_LEN_MAX) {
    fprintf(stderr, "sendTCPLSR: Options too large\n");
    return(-1); 
  }
  
  packetSize = IP_HDR_LEN + TCP_HDR_LEN + IPoptionSize;
  if((packetBuf = (uint8_t *)malloc(packetSize)) == NULL) {
    fprintf(stderr, "sendLSR: malloc returned NULL\n");
    return(-1);
  }
  bzero((char *)packetBuf, packetSize);

  fillIPHeader(packetBuf, settings);
  fillIPOptLSR(packetBuf+IP_HDR_LEN, settings);
  fillTCPHeader(packetBuf+IP_HDR_LEN+IPoptionSize, settings);

  ip_checksum(packetBuf, packetSize);
  
  ip_send(ip, packetBuf, packetSize);

  ip_close(ip);
  
  free(packetBuf);
  
  return(0);
}

int main(int argc, char **argv) {
  struct packetInfo settings;
  struct addr tmpAddr, myAddr;
  pcap_t *pcapPtr;
  int acceptTo = 0;
  int acceptThrough = 0;
  int c;
  int i;
  uint32_t tempIp;
  int sourcePortDefined = 0;
  int destPort = 80;
  int sourceIPDefined = 0;
  int usingSilent = 0;
  int scanType = TYPE_BOTH;
  unsigned int tempRand;

  if (argc < 2) usage(argv[0]);

  if(!(getuid() == 0)) {
    printf("Gotta be root to run %s\n", argv[0]);
    return(-1);
  }

  /*
   * s: source port (default random)
   * S: source IP address (default random)
   * d: destination port (default 80)
   * i: "silent" IP. i.e. you're not using your real IP, but one you
   *    can sniff anyways.
   * t: Type of scan, to or through or both. Default both
   */

  settings.silentIP = 0;

  while((c=getopt(argc, argv, "S:s:p:i:t:h")) != -1) {
    switch (c) {
    case 'S':
      sourceIPDefined++;
      if ((addr_pton(optarg, &tmpAddr)) < 0) {
	fprintf(stderr, "Cannot resolve source IP %s\n", optarg);
	return(-1);
      } else {
	settings.ipHops[0] = tmpAddr.addr_ip;
      }
      break;

    case 'h':
      usage(argv[0]);
      break;      
      
    case 's':
      sourcePortDefined = atoi(optarg);
      if((sourcePortDefined < 0) || (sourcePortDefined > TCP_PORT_MAX)) {
	fprintf(stderr, "Invalid source port: %d\n", sourcePortDefined);
	return(-1);
      }
      settings.sourcePort = sourcePortDefined;
      break;

    case 'p':
      destPort = atoi(optarg);
      if((destPort < 0) || (destPort > TCP_PORT_MAX)) {
	fprintf(stderr, "Invalid dest port: %d\n", destPort);
	return(-1);
      }
      break;
      
    case 'i':
      settings.silentIP++;
      if ((addr_pton(optarg, &myAddr)) < 0) {
	fprintf(stderr, "Cannot resolve silent IP%s\n", optarg);
	return(-1);
      } 
      break;

    case 't':
      if(strstr(optarg, "to")) scanType = TYPE_TO; 
      else if (strstr(optarg, "b")) scanType = TYPE_BOTH;
      else if (strstr(optarg, "th")) scanType = TYPE_THROUGH;
      else {
	fprintf(stderr, "Invalid scan type. valid:(to|through|both)\n");
	return(-1);
      }

      break;
    }
  }

  settings.destPort = destPort;

  if((pcapPtr = initListen(&settings, &myAddr)) == NULL) 
    return(-1); /* I think I've already got error messages on all the ways 
		 * you can bomb out */
  
  /* OK, we now nead to loop over each IP or CIDR block, 
   * and scramble the options, send packets, and listen 
   * I'm gonna do this with just IPs for now, CIDR later.
   */

  for(i = optind; i < argc; i++) {
    if ((addr_pton(argv[i], &tmpAddr)) < 0) {
      fprintf(stderr, "Cannot resolve dest IP %s\n", optarg);
      return(-1);
    }

    if (sourceIPDefined == 0) 
      fillBufRandomly(&(settings.ipHops[0]), IP_ADDR_LEN);

    settings.hopCount = 3;

    if((scanType == TYPE_TO) || (scanType == TYPE_BOTH)) {
      if (sourcePortDefined == 0)
	fillBufRandomly(&(settings.sourcePort), 2);
      settings.type = TYPE_TO;
      settings.ipHops[2] = tmpAddr.addr_ip;
      settings.myHop = 1;
      settings.ipHops[settings.myHop] = myAddr.addr_ip;
      settings.currentHop = 2;
      if(sendLSR(&settings) < 0) {
	fprintf(stderr,"Error in sendLSR\n");
      } else {
	if(getResponse(pcapPtr, &settings) == 1) 
	  printf("%s reverses LSR traffic to it\n", argv[i]);
	else
	  printf("%s does not reverse LSR traffic to it\n", argv[i]);
      }
    }
    if((scanType == TYPE_THROUGH) || (scanType == TYPE_BOTH)) {
      if (sourcePortDefined == 0)
	fillBufRandomly(&(settings.sourcePort), 2);
      settings.type = TYPE_THROUGH;
      settings.myHop = 2;
      settings.ipHops[settings.myHop] = myAddr.addr_ip;
      settings.ipHops[1] = tmpAddr.addr_ip;
      settings.currentHop = 1;
      if(sendLSR(&settings) < 0) {
	fprintf(stderr,"Error in sendLSR\n");
      } else {
	if(getResponse(pcapPtr, &settings) == 1) 
	  printf("%s forwards LSR traffic through it\n", argv[i]);
	else
	  printf("%s does not forward LSR traffic through it\n", argv[i]);
      }
    }
  }
}

/* TODO:
 *  (also expand # of IP addrs you can send)
*/
