#include <dnet.h>
#include "lsrscan.h"

int fillIPHeader(uint8_t *headerBuf, struct packetInfo *settings) {
  struct ip_hdr *ipHeader;
  int optionSize;
  int optionWords;
  uint16_t length;

  optionSize = LSROVERHEAD+(settings->hopCount-2)*IP_ADDR_LEN;
  if (optionSize%4)
    optionSize += (4-(optionSize%4)); /* Pad out to word boundary */
  optionWords = optionSize / 4;

  length = IP_HDR_LEN + optionWords*4 + TCP_HDR_LEN;
  
  ipHeader = (struct ip_hdr *)headerBuf;
  ipHeader->ip_v = 4;
  ipHeader->ip_hl = 5 + optionWords;
  ipHeader->ip_len = htons(length);
  fillBufRandomly(&(ipHeader->ip_id), 2);
  ipHeader->ip_ttl = 255;
  ipHeader->ip_p = IPPROTO_TCP;
  ipHeader->ip_src = settings->ipHops[0];
  ipHeader->ip_dst = settings->ipHops[settings->currentHop];
  
  return(0);
}

/* remember that the current destination IP address and the
 * source address are not in the LSR field. See TCP/IP vol2
 * page 255 
 */

int fillIPOptLSR(uint8_t *buffer, struct packetInfo *settings){
  uint8_t i;
  
  *buffer = IP_OPT_LSRR;
  buffer++;
  *buffer = LSROVERHEAD+(IP_ADDR_LEN*(settings->hopCount - 2));
  buffer++;
  *buffer = (settings->currentHop*IP_ADDR_LEN);
  buffer++;
  
  for(i = 1; i < (settings->hopCount); i++) {
    if (!(i == settings->currentHop)) {
      memcpy (buffer, &(settings->ipHops[i]), IP_ADDR_LEN);
      buffer+=IP_ADDR_LEN;
    }
  }
  return(0);
}

int fillTCPHeader(uint8_t *buffer, struct packetInfo *settings) {
   struct tcp_hdr *tcpHeader;
  
  tcpHeader = (struct tcp_hdr *)buffer;
  tcpHeader->th_sport = htons(settings->sourcePort);
  tcpHeader->th_dport = htons(settings->destPort);
  fillBufRandomly(&(tcpHeader->th_seq), 4);
  settings->sequenceNum = tcpHeader->th_seq;
  tcpHeader->th_off = 5;
  tcpHeader->th_flags = TH_SYN;
  tcpHeader->th_win = htons(31856); /* FIX ME!!! Do I care about this? */
  
  return(0);
}
